/* 
 * Copyright (C) 2002 Laird Breyer
 *  
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 * 
 * Author:   Laird Breyer <laird@lbreyer.com>
 */

/* maximum number of categories we can handle simultaneously */
#define MAX_CAT 64
/* percentage of hash we use */
#define HASH_FULL 95
/* size of hash in bits */
#define MAX_TOKEN_BITS 26
/* maximum size of a token, beyond that rest is ignored */
#define MAX_TOKEN_LEN 100
/* for line filtering: maximum number of tokens allowed on a single line */
#define MAX_TOKEN_LINE_STACK 128 
#define BUFLEN 1024
/* alphabet size */
#define ASIZE 256

#define OPTION_CLASSIFY     1
#define OPTION_LEARN        2
#define OPTION_TITLE        3
#define OPTION_CUTOFF       4
#define OPTION_VERBOSE      5
#define INPUT_FROM_CMDLINE  6
#define OPTION_SCORES       7
#define OPTION_POSTERIOR    8
#define OPTION_IDENTIFY     9
#define OPTION_FILTER       10
#define OPTION_REFMODEL     11
#define OPTION_TEXT_FORMAT  12
#define OPTION_MBOX_FORMAT  13
#define OPTION_XML          14
#define OPTION_DEBUG        15
#define OPTION_I18N         16
#define OPTION_NOREGEX      17
#define OPTION_CASEN        18


typedef long int re_bitfield;
/* maximum number of regular expressions we can handle */
#define MAX_RE 32
/* maximum number of tagged subexpressions we can handle for each regex */
#define MAX_SUBMATCH 10

/* used in hash code */
#define FILLEDP(a) ((a)->id[0] || (a)->id[1] || (a)->id[2])

#define EQUALP(a,b) (((a)[0] == (char)((b) & 255)) && \
                     ((a)[1] == (char)(((b)>>8) & 255)) && \
                     ((a)[2] == (char)(((b)>>16) & 255)))

#define SET(a,b) { (a)[0] = ((b) & 255); \
                   (a)[1] = (((b)>>8) & 255); \
                   (a)[2] = (((b)>>16) & 255); }

/* data structures */

typedef struct {
  char id[3];
  int count;
} h_item;

typedef struct {
  long int max_tokens;
  short int max_token_bits;
  long int full_token_count;
  long int unique_token_count;
  h_item *hash;
  int cached;
  h_item *stack[MAX_TOKEN_LINE_STACK];
  int stack_top;
} Empirical;

typedef struct {
  char id[3]; /* suitable for up to 16 million items or so */
#if defined DIGITIZE_LAMBDA
  short int lam;
#else
  float lam;
#endif
} c_item;

typedef struct {
  char *filename;
  long int unique_token_count;
  long int full_token_count;
  long int max_tokens;
  short int max_token_bits;
  re_bitfield retype;
  double logZ;
  double divergence;
  double score;
  c_item *hash;
#if defined DIGITIZE_DIGRAMS
  short int dig[ASIZE][ASIZE];
#else
  float dig[ASIZE][ASIZE];
#endif
} Category;

typedef struct {
  char id[3];
  int  count;
  float lam;
  float ref;
} l_item;

typedef struct {
  char *filename;
  FILE *tmp;
  long int max_tokens;
  short int max_token_bits;
  long int full_token_count;
  long int unique_token_count;
  double logZ;
  double divergence;
  l_item *hash;
  float dig[ASIZE][ASIZE];
} Learner;

typedef struct {
  double alpha;
  double u[ASIZE];
} Dirichlet;

typedef struct {
  regex_t regex;
#if defined HAVE_LIBBOOST_REGEX
  wchar_t *string;
#else 
  char *string;
#endif
} Regex;

typedef struct {
  enum { UNDEF, HEADER, BODY} state;
  int prev_line_empty;
  int mime_type;
  int checked_content_type;
} MBOX_State;

typedef struct {
  enum {TEXT, TAG, COMMENT, SPECIAL} state;
} XML_State;
